package java.awt.datatransfer;

import java.lang.ref.SoftReference;
import java.util.Objects;
import java.util.ArrayList;
import java.util.List;
import java.util.Iterator;
import sun.awt.datatransfer.DataTransferer;
import java.net.URL;
import java.awt.Toolkit;
import java.security.AccessController;
import java.io.IOException;
import java.net.MalformedURLException;
import java.io.InputStreamReader;
import java.io.File;
import java.security.PrivilegedAction;
import java.io.BufferedReader;
import java.util.HashSet;
import java.util.HashMap;
import sun.awt.AppContext;
import java.util.Collections;
import java.util.Set;
import java.util.LinkedHashSet;
import java.util.Map;

public final class SystemFlavorMap implements FlavorMap, FlavorTable
{
    private static String JavaMIME;
    private static final Object FLAVOR_MAP_KEY;
    private static final String keyValueSeparators = "=: \t\r\n\f";
    private static final String strictKeyValueSeparators = "=:";
    private static final String whiteSpaceChars = " \t\r\n\f";
    private static final String[] UNICODE_TEXT_CLASSES;
    private static final String[] ENCODED_TEXT_CLASSES;
    private static final String TEXT_PLAIN_BASE_TYPE = "text/plain";
    private static final String HTML_TEXT_BASE_TYPE = "text/html";
    private final Map<String, LinkedHashSet<DataFlavor>> nativeToFlavor;
    private final Map<DataFlavor, LinkedHashSet<String>> flavorToNative;
    private Map<String, LinkedHashSet<String>> textTypeToNative;
    private boolean isMapInitialized;
    private final SystemFlavorMap.SoftCache<DataFlavor, String> nativesForFlavorCache;
    private final SystemFlavorMap.SoftCache<String, DataFlavor> flavorsForNativeCache;
    private Set<Object> disabledMappingGenerationKeys;
    private static final String[] htmlDocumntTypes;
    
    private Map<String, LinkedHashSet<DataFlavor>> getNativeToFlavor() {
        if (!this.isMapInitialized) {
            this.initSystemFlavorMap();
        }
        return this.nativeToFlavor;
    }
    
    private synchronized Map<DataFlavor, LinkedHashSet<String>> getFlavorToNative() {
        if (!this.isMapInitialized) {
            this.initSystemFlavorMap();
        }
        return this.flavorToNative;
    }
    
    private synchronized Map<String, LinkedHashSet<String>> getTextTypeToNative() {
        if (!this.isMapInitialized) {
            this.initSystemFlavorMap();
            this.textTypeToNative = Collections.unmodifiableMap(this.textTypeToNative);
        }
        return this.textTypeToNative;
    }
    
    public static FlavorMap getDefaultFlavorMap() {
        AppContext appContext = AppContext.getAppContext();
        FlavorMap value = (FlavorMap)appContext.get(SystemFlavorMap.FLAVOR_MAP_KEY);
        if (value == null) {
            value = new SystemFlavorMap();
            appContext.put(SystemFlavorMap.FLAVOR_MAP_KEY, value);
            return value;
        }
        return value;
    }
    
    private SystemFlavorMap() {
        this.nativeToFlavor = new HashMap();
        this.flavorToNative = new HashMap();
        this.textTypeToNative = new HashMap();
        this.isMapInitialized = false;
        this.nativesForFlavorCache = new SystemFlavorMap.SoftCache();
        this.flavorsForNativeCache = new SystemFlavorMap.SoftCache();
        this.disabledMappingGenerationKeys = new HashSet();
    }
    
    private void initSystemFlavorMap() {
        if (this.isMapInitialized) {
            return;
        }
        this.isMapInitialized = true;
        BufferedReader bufferedReader = (BufferedReader)AccessController.doPrivileged(new PrivilegedAction<BufferedReader>() {
            @Override
            public BufferedReader run() {
                String string = System.getProperty("java.home") + File.separator + "lib" + File.separator + "flavormap.properties";
                try {
                    return new BufferedReader(new InputStreamReader(new File(string).toURI().toURL().openStream(), "ISO-8859-1"));
                }
                catch (MalformedURLException obj) {
                    System.err.println("MalformedURLException:" + obj + " while loading default flavormap.properties file:" + string);
                }
                catch (IOException obj2) {
                    System.err.println("IOException:" + obj2 + " while loading default flavormap.properties file:" + string);
                }
                return null;
            }
        });
        String str = (String)AccessController.doPrivileged(new PrivilegedAction<String>() {
            @Override
            public String run() {
                return Toolkit.getProperty("AWT.DnD.flavorMapFileURL", null);
            }
        });
        if (bufferedReader != null) {
            try {
                this.parseAndStoreReader(bufferedReader);
            }
            catch (IOException obj) {
                System.err.println("IOException:" + obj + " while parsing default flavormap.properties file");
            }
        }
        BufferedReader bufferedReader2 = null;
        if (str != null) {
            try {
                bufferedReader2 = new BufferedReader(new InputStreamReader(new URL(str).openStream(), "ISO-8859-1"));
            }
            catch (MalformedURLException obj2) {
                System.err.println("MalformedURLException:" + obj2 + " while reading AWT.DnD.flavorMapFileURL:" + str);
            }
            catch (IOException obj3) {
                System.err.println("IOException:" + obj3 + " while reading AWT.DnD.flavorMapFileURL:" + str);
            }
            catch (SecurityException ex) {}
        }
        if (bufferedReader2 != null) {
            try {
                this.parseAndStoreReader(bufferedReader2);
            }
            catch (IOException obj4) {
                System.err.println("IOException:" + obj4 + " while parsing AWT.DnD.flavorMapFileURL");
            }
        }
        return;
    }
    
    private void parseAndStoreReader(BufferedReader bufferedReader) throws IOException {
        while (true) {
            String s = bufferedReader.readLine();
            if (s == null) {
                break;
            }
            if (s.length() > 0) {
                int char1 = s.charAt(0);
                if (char1 != '#' && char1 != '!') {
                    while (this.continueLine(s)) {
                        String line = bufferedReader.readLine();
                        if (line == null) {
                            line = "";
                        }
                        String substring = s.substring(0, s.length() - 1);
                        int n;
                        for (n = 0; n < line.length() && " \t\r\n\f".indexOf((int)line.charAt(n)) != -1; ++n) {}
                        s = substring + line.substring(n, line.length());
                    }
                    int length;
                    int n2;
                    for (length = s.length(), n2 = 0; n2 < length && " \t\r\n\f".indexOf((int)s.charAt(n2)) != -1; ++n2) {}
                    if (n2 == length) {
                        continue;
                    }
                    int i;
                    for (i = n2; i < length; ++i) {
                        int char2 = s.charAt(i);
                        if (char2 == '\\') {
                            ++i;
                        }
                        else if ("=: \t\r\n\f".indexOf(char2) != -1) {
                            break;
                        }
                    }
                    for (; i < length && " \t\r\n\f".indexOf((int)s.charAt(i)) != -1; ++i) {}
                    if (i < length && "=:".indexOf((int)s.charAt(i)) != -1) {
                        ++i;
                    }
                    while (i < length && " \t\r\n\f".indexOf((int)s.charAt(i)) != -1) {
                        ++i;
                    }
                    String substring2 = s.substring(n2, i);
                    String s2 = (i < length) ? s.substring(i, length) : "";
                    String loadConvert = this.loadConvert(substring2);
                    String s3 = this.loadConvert(s2);
                    try {
                        MimeType o = new MimeType(s3);
                        if ("text".equals(o.getPrimaryType())) {
                            String parameter = o.getParameter("charset");
                            if (DataTransferer.doesSubtypeSupportCharset(o.getSubType(), parameter)) {
                                DataTransferer instance = DataTransferer.getInstance();
                                if (instance != null) {
                                    instance.registerTextFlavorProperties(loadConvert, parameter, o.getParameter("eoln"), o.getParameter("terminators"));
                                }
                            }
                            o.removeParameter("charset");
                            o.removeParameter("class");
                            o.removeParameter("eoln");
                            o.removeParameter("terminators");
                            s3 = o.toString();
                        }
                    }
                    catch (MimeTypeParseException ex) {
                        ex.printStackTrace();
                        continue;
                    }
                    Object e;
                    try {
                        e = new DataFlavor(s3);
                    }
                    catch (Exception ex2) {
                        try {
                            e = new DataFlavor(s3, null);
                        }
                        catch (Exception ex3) {
                            ex3.printStackTrace();
                        }
                    }
                    LinkedHashSet o2 = new LinkedHashSet();
                    o2.add(e);
                    if ("text".equals(e.getPrimaryType())) {
                        o2.addAll(convertMimeTypeToDataFlavors(s3));
                        this.store(e.mimeType.getBaseType(), loadConvert, this.getTextTypeToNative());
                    }
                    for (DataFlavor dataFlavor : o2) {
                        this.store(dataFlavor, loadConvert, this.getFlavorToNative());
                        this.store(loadConvert, dataFlavor, this.getNativeToFlavor());
                    }
                }
                else {
                    continue;
                }
            }
            else {
                continue;
            }
        }
    }
    
    private boolean continueLine(String s) {
        int n = 0;
        int n2 = s.length() - 1;
        while (n2 >= 0 && s.charAt(n2--) == '\\') {
            ++n;
        }
        return n % 2 == 1;
    }
    
    private String loadConvert(String s) {
        int length = s.length();
        StringBuilder sb = new StringBuilder(length);
        int i = 0;
        while (i < length) {
            int char1 = s.charAt(i++);
            if (char1 == '\\') {
                int char2 = s.charAt(i++);
                if (char2 == 'u') {
                    int n = 0;
                    for (int j = 0; j < 4; ++j) {
                        int char3 = s.charAt(i++);
                        switch (char3) {
                            case 48:
                            case 49:
                            case 50:
                            case 51:
                            case 52:
                            case 53:
                            case 54:
                            case 55:
                            case 56:
                            case 57: {
                                n = (n << 4) + char3 - 48;
                                break;
                            }
                            case 97:
                            case 98:
                            case 99:
                            case 100:
                            case 101:
                            case 102: {
                                n = (n << 4) + 10 + char3 - 97;
                                break;
                            }
                            case 65:
                            case 66:
                            case 67:
                            case 68:
                            case 69:
                            case 70: {
                                n = (n << 4) + 10 + char3 - 65;
                                break;
                            }
                            default: {
                                throw new IllegalArgumentException("Malformed \\uxxxx encoding.");
                            }
                        }
                    }
                    sb.append((char)n);
                }
                else {
                    if (char2 == 't') {
                        char2 = '\t';
                    }
                    else if (char2 == 'r') {
                        char2 = '\r';
                    }
                    else if (char2 == 'n') {
                        char2 = '\n';
                    }
                    else if (char2 == 'f') {
                        char2 = '\f';
                    }
                    sb.append((char)char2);
                }
            }
            else {
                sb.append((char)char1);
            }
        }
        return sb.toString();
    }
    
    private <H, L> void store(H h, L l, Map<H, LinkedHashSet<L>> map) {
        LinkedHashSet<L> set = (LinkedHashSet)map.get(h);
        if (set == null) {
            set = new LinkedHashSet(1);
            map.put(h, set);
        }
        if (!set.contains(l)) {
            set.add(l);
        }
    }
    
    private LinkedHashSet<DataFlavor> nativeToFlavorLookup(String s) {
        Object c = (LinkedHashSet<DataFlavor>)this.getNativeToFlavor().get(s);
        if (s != null && !this.disabledMappingGenerationKeys.contains(s)) {
            DataTransferer instance = DataTransferer.getInstance();
            if (instance != null) {
                LinkedHashSet<DataFlavor> platformMappingsForNative = instance.getPlatformMappingsForNative(s);
                if (!platformMappingsForNative.isEmpty()) {
                    if (c != null) {
                        platformMappingsForNative.addAll(c);
                    }
                    c = platformMappingsForNative;
                }
            }
        }
        if (c == null && isJavaMIMEType(s)) {
            String decodeJavaMIMEType = decodeJavaMIMEType(s);
            Object e = null;
            try {
                e = new DataFlavor(decodeJavaMIMEType);
            }
            catch (Exception ex) {
                System.err.println("Exception \"" + ex.getClass().getName() + ": " + ex.getMessage() + "\"while constructing DataFlavor for: " + decodeJavaMIMEType);
            }
            if (e != null) {
                c = new LinkedHashSet(1);
                this.getNativeToFlavor().put(s, c);
                c.add(e);
                this.flavorsForNativeCache.remove(s);
                LinkedHashSet set = (LinkedHashSet<String>)this.getFlavorToNative().get(e);
                if (set == null) {
                    set = new LinkedHashSet(1);
                    this.getFlavorToNative().put(e, set);
                }
                set.add(s);
                this.nativesForFlavorCache.remove(e);
            }
        }
        return (c != null) ? c : new LinkedHashSet(0);
    }
    
    private LinkedHashSet<String> flavorToNativeLookup(DataFlavor dataFlavor, boolean b) {
        Object o = (LinkedHashSet<String>)this.getFlavorToNative().get(dataFlavor);
        if (dataFlavor != null && !this.disabledMappingGenerationKeys.contains(dataFlavor)) {
            DataTransferer instance = DataTransferer.getInstance();
            if (instance != null) {
                LinkedHashSet<String> platformMappingsForFlavor = instance.getPlatformMappingsForFlavor(dataFlavor);
                if (!platformMappingsForFlavor.isEmpty()) {
                    if (o != null) {
                        platformMappingsForFlavor.addAll(o);
                    }
                    o = platformMappingsForFlavor;
                }
            }
        }
        if (o == null) {
            if (b) {
                String encodeDataFlavor = encodeDataFlavor(dataFlavor);
                o = new LinkedHashSet(1);
                this.getFlavorToNative().put(dataFlavor, o);
                o.add(encodeDataFlavor);
                LinkedHashSet set = (LinkedHashSet<DataFlavor>)this.getNativeToFlavor().get(encodeDataFlavor);
                if (set == null) {
                    set = new LinkedHashSet(1);
                    this.getNativeToFlavor().put(encodeDataFlavor, set);
                }
                set.add(dataFlavor);
                this.nativesForFlavorCache.remove(dataFlavor);
                this.flavorsForNativeCache.remove(encodeDataFlavor);
            }
            else {
                o = new LinkedHashSet(0);
            }
        }
        return new LinkedHashSet(o);
    }
    
    @Override
    public synchronized List<String> getNativesForFlavor(DataFlavor dataFlavor) {
        LinkedHashSet<String> check = this.nativesForFlavorCache.check(dataFlavor);
        if (check != null) {
            return new ArrayList(check);
        }
        LinkedHashSet c;
        if (dataFlavor == null) {
            c = new LinkedHashSet(this.getNativeToFlavor().keySet());
        }
        else if (this.disabledMappingGenerationKeys.contains(dataFlavor)) {
            c = this.flavorToNativeLookup(dataFlavor, false);
        }
        else if (DataTransferer.isFlavorCharsetTextType(dataFlavor)) {
            c = new LinkedHashSet(0);
            if ("text".equals(dataFlavor.getPrimaryType())) {
                LinkedHashSet<Object> c2 = (LinkedHashSet)this.getTextTypeToNative().get(dataFlavor.mimeType.getBaseType());
                if (c2 != null) {
                    c.addAll(c2);
                }
            }
            LinkedHashSet<Object> c3 = (LinkedHashSet)this.getTextTypeToNative().get("text/plain");
            if (c3 != null) {
                c.addAll(c3);
            }
            if (c.isEmpty()) {
                c = this.flavorToNativeLookup(dataFlavor, true);
            }
            else {
                c.addAll(this.flavorToNativeLookup(dataFlavor, false));
            }
        }
        else if (DataTransferer.isFlavorNoncharsetTextType(dataFlavor)) {
            c = (LinkedHashSet)this.getTextTypeToNative().get(dataFlavor.mimeType.getBaseType());
            if (c == null || c.isEmpty()) {
                c = this.flavorToNativeLookup(dataFlavor, true);
            }
            else {
                c.addAll(this.flavorToNativeLookup(dataFlavor, false));
            }
        }
        else {
            c = this.flavorToNativeLookup(dataFlavor, true);
        }
        this.nativesForFlavorCache.put(dataFlavor, c);
        return new ArrayList(c);
    }
    
    @Override
    public synchronized List<DataFlavor> getFlavorsForNative(String s) {
        LinkedHashSet<DataFlavor> check = this.flavorsForNativeCache.check(s);
        if (check != null) {
            return new ArrayList(check);
        }
        LinkedHashSet<DataFlavor> c = new LinkedHashSet();
        if (s == null) {
            Iterator<String> iterator2 = this.getNativesForFlavor(null).iterator();
            while (iterator2.hasNext()) {
                c.addAll(this.getFlavorsForNative((String)iterator2.next()));
            }
        }
        else {
            LinkedHashSet<DataFlavor> nativeToFlavorLookup = this.nativeToFlavorLookup(s);
            if (this.disabledMappingGenerationKeys.contains(s)) {
                return new ArrayList(nativeToFlavorLookup);
            }
            for (DataFlavor e : this.nativeToFlavorLookup(s)) {
                c.add(e);
                if ("text".equals(e.getPrimaryType())) {
                    c.addAll(convertMimeTypeToDataFlavors(e.mimeType.getBaseType()));
                }
                else {
                    continue;
                }
            }
        }
        this.flavorsForNativeCache.put(s, c);
        return new ArrayList(c);
    }
    
    private static Set<DataFlavor> convertMimeTypeToDataFlavors(String anObject) {
        LinkedHashSet<DataFlavor> set = new LinkedHashSet();
        String subType = null;
        try {
            subType = new MimeType(anObject).getSubType();
        }
        catch (MimeTypeParseException ex) {}
        if (DataTransferer.doesSubtypeSupportCharset(subType, null)) {
            if ("text/plain".equals(anObject)) {
                set.add(DataFlavor.stringFlavor);
            }
            String[] unicode_TEXT_CLASSES = SystemFlavorMap.UNICODE_TEXT_CLASSES;
            for (int length2 = unicode_TEXT_CLASSES.length, j = 0; j < length2; ++j) {
                for (String s : handleHtmlMimeTypes(anObject, anObject + ";charset=Unicode;class=" + unicode_TEXT_CLASSES[j])) {
                    Object o2 = null;
                    try {
                        o2 = new DataFlavor(s);
                    }
                    catch (ClassNotFoundException ex3) {}
                    set.add(o2);
                }
            }
            for (String str2 : DataTransferer.standardEncodings()) {
                String[] encoded_TEXT_CLASSES2 = SystemFlavorMap.ENCODED_TEXT_CLASSES;
                for (int length3 = encoded_TEXT_CLASSES2.length, k = 0; k < length3; ++k) {
                    for (String s2 : handleHtmlMimeTypes(anObject, anObject + ";charset=" + str2 + ";class=" + encoded_TEXT_CLASSES2[k])) {
                        Object plainTextFlavor = null;
                        try {
                            plainTextFlavor = new DataFlavor(s2);
                            if (plainTextFlavor.equals(DataFlavor.plainTextFlavor)) {
                                plainTextFlavor = DataFlavor.plainTextFlavor;
                            }
                        }
                        catch (ClassNotFoundException ex4) {}
                        set.add(plainTextFlavor);
                    }
                }
            }
            if ("text/plain".equals(anObject)) {
                set.add(DataFlavor.plainTextFlavor);
            }
        }
        else {
            for (String str : SystemFlavorMap.ENCODED_TEXT_CLASSES) {
                Object o = null;
                try {
                    o = new DataFlavor(anObject + ";class=" + str);
                }
                catch (ClassNotFoundException ex2) {}
                set.add(o);
            }
        }
        return set;
    }
    
    private static LinkedHashSet<String> handleHtmlMimeTypes(String anObject, String s) {
        LinkedHashSet<String> set = new LinkedHashSet();
        if ("text/html".equals(anObject)) {
            String[] htmlDocumntTypes = SystemFlavorMap.htmlDocumntTypes;
            for (int length = htmlDocumntTypes.length, i = 0; i < length; ++i) {
                set.add(s + ";document=" + htmlDocumntTypes[i]);
            }
            return set;
        }
        set.add(s);
        return set;
    }
    
    @Override
    public synchronized Map<DataFlavor, String> getNativesForFlavors(DataFlavor[] array) {
        if (array == null) {
            List<DataFlavor> flavorsForNative = this.getFlavorsForNative(null);
            array = new DataFlavor[flavorsForNative.size()];
            flavorsForNative.toArray(array);
        }
        HashMap o = new HashMap(array.length, 1.0f);
        for (DataFlavor dataFlavor : array) {
            List<String> nativesForFlavor = this.getNativesForFlavor(dataFlavor);
            o.put(dataFlavor, nativesForFlavor.isEmpty() ? null : ((Object)nativesForFlavor.get(0)));
        }
        return o;
    }
    
    @Override
    public synchronized Map<String, DataFlavor> getFlavorsForNatives(String[] array) {
        if (array == null) {
            List<String> nativesForFlavor = this.getNativesForFlavor(null);
            array = new String[nativesForFlavor.size()];
            nativesForFlavor.toArray(array);
        }
        HashMap o = new HashMap(array.length, 1.0f);
        for (String s : array) {
            List<DataFlavor> flavorsForNative = this.getFlavorsForNative(s);
            o.put(s, flavorsForNative.isEmpty() ? null : ((Object)flavorsForNative.get(0)));
        }
        return o;
    }
    
    public synchronized void addUnencodedNativeForFlavor(DataFlavor obj, String s) {
        Objects.requireNonNull(s, "Null native not permitted");
        Objects.requireNonNull(obj, "Null flavor not permitted");
        LinkedHashSet<String> set = (LinkedHashSet)this.getFlavorToNative().get(obj);
        if (set == null) {
            set = new LinkedHashSet(1);
            this.getFlavorToNative().put(obj, set);
        }
        set.add(s);
        this.nativesForFlavorCache.remove(obj);
    }
    
    public synchronized void setNativesForFlavor(DataFlavor obj, String[] obj2) {
        Objects.requireNonNull(obj2, "Null natives not permitted");
        Objects.requireNonNull(obj, "Null flavors not permitted");
        this.getFlavorToNative().remove(obj);
        for (int length = obj2.length, i = 0; i < length; ++i) {
            this.addUnencodedNativeForFlavor(obj, obj2[i]);
        }
        this.disabledMappingGenerationKeys.add(obj);
        this.nativesForFlavorCache.remove(obj);
    }
    
    public synchronized void addFlavorForUnencodedNative(String obj, DataFlavor dataFlavor) {
        Objects.requireNonNull(obj, "Null native not permitted");
        Objects.requireNonNull(dataFlavor, "Null flavor not permitted");
        LinkedHashSet<DataFlavor> set = (LinkedHashSet)this.getNativeToFlavor().get(obj);
        if (set == null) {
            set = new LinkedHashSet(1);
            this.getNativeToFlavor().put(obj, set);
        }
        set.add(dataFlavor);
        this.flavorsForNativeCache.remove(obj);
    }
    
    public synchronized void setFlavorsForNative(String obj, DataFlavor[] obj2) {
        Objects.requireNonNull(obj, "Null native not permitted");
        Objects.requireNonNull(obj2, "Null flavors not permitted");
        this.getNativeToFlavor().remove(obj);
        for (int length = obj2.length, i = 0; i < length; ++i) {
            this.addFlavorForUnencodedNative(obj, obj2[i]);
        }
        this.disabledMappingGenerationKeys.add(obj);
        this.flavorsForNativeCache.remove(obj);
    }
    
    public static String encodeJavaMIMEType(String str) {
        return (str != null) ? (SystemFlavorMap.JavaMIME + str) : null;
    }
    
    public static String encodeDataFlavor(DataFlavor dataFlavor) {
        return (dataFlavor != null) ? encodeJavaMIMEType(dataFlavor.getMimeType()) : null;
    }
    
    public static boolean isJavaMIMEType(String s) {
        return s != null && s.startsWith(SystemFlavorMap.JavaMIME, 0);
    }
    
    public static String decodeJavaMIMEType(String s) {
        return isJavaMIMEType(s) ? s.substring(SystemFlavorMap.JavaMIME.length(), s.length()).trim() : null;
    }
    
    public static DataFlavor decodeDataFlavor(String s) throws ClassNotFoundException {
        String decodeJavaMIMEType = decodeJavaMIMEType(s);
        return (decodeJavaMIMEType != null) ? new DataFlavor(decodeJavaMIMEType) : null;
    }
    
    static {
        SystemFlavorMap.JavaMIME = "JAVA_DATAFLAVOR:";
        FLAVOR_MAP_KEY = new Object();
        UNICODE_TEXT_CLASSES = new String[] { "java.io.Reader", "java.lang.String", "java.nio.CharBuffer", "\"[C\"" };
        ENCODED_TEXT_CLASSES = new String[] { "java.io.InputStream", "java.nio.ByteBuffer", "\"[B\"" };
        htmlDocumntTypes = new String[] { "all", "selection", "fragment" };
    }
    
    private static final class SoftCache<K, V>
    {
        Map<K, SoftReference<LinkedHashSet<V>>> cache;
        
        public void put(K k, LinkedHashSet<V> referent) {
            if (this.cache == null) {
                this.cache = new HashMap(1);
            }
            this.cache.put(k, new SoftReference(referent));
        }
        
        public void remove(K k) {
            if (this.cache == null) {
                return;
            }
            this.cache.remove(null);
            this.cache.remove(k);
            return;
        }
        
        public LinkedHashSet<V> check(K k) {
            if (this.cache == null) {
                return null;
            }
            SoftReference<Object> softReference = (SoftReference)this.cache.get(k);
            if (softReference != null) {
                return (LinkedHashSet)softReference.get();
            }
            return null;
        }
    }
}
